
import os.path
import sqlite3
from collections import OrderedDict
from sqlite3 import OperationalError
from sqlite3 import IntegrityError
from datetime import datetime as dt, timedelta, timezone 

from logger_util import LoggerUtil


class DBUtil:
    def __init__(self, file_path: str = ''):
        self.conn = None
        self.logger = None
        self.filename = ''

        try:
            if not os.path.exists(file_path.rsplit('/', 1)[0]):
                os.mkdir(file_path.rsplit('/', 1)[0])
                
            conn = sqlite3.connect(file_path)
            conn.execute('PRAGMA journal_mode=memory')
            conn.execute('PRAGMA auto_vacuum=full')
            self.conn = conn
            self.init_logger(file_path)
        except OperationalError as e:
            print("SQLite3 database file does not ready: {}".format(e))

    def init_logger(self, file_path: str = ''):
        log_path = os.path.join(file_path.rsplit("/", 2)[0], 'result')
        self.logger = LoggerUtil(log_path, log_ext="sql", use_timestamp=False)
        self.filename = file_path.rsplit("/", 1)[1].rsplit(".")[0]
        self.logger.log_info(self.filename,
                             "-- created at {}".format(
                                 dt.now(
                                     timezone(timedelta(hours=9), name="KST")
                                 ).strftime("%Y-%m-%d %H:%M:%S.%f %Z")
                             )
                             )

    def get_connection(self):
        return self.conn

    @staticmethod
    def get_type(type_name: str) -> str:
        type_dict = {
            "int": "INTEGER",
            "str": "TEXT",
            "float": "REAL",
            "bool": "NUMERIC"
        }
        return type_dict[type_name]

    @staticmethod
    def get_default_val(type_name: str) -> str:
        valstr = ''

        if type_name in ['int', 'bool']:
            valstr = '0'
        elif type_name == 'float':
            valstr = '0.0'
        elif type_name == 'str':
            valstr = '\'\''

        return 'DEFAULT {}'.format(valstr)

    @staticmethod
    def get_primary_key_field_name(table_name: str) -> str:
        tnarr = table_name.split('_')
        primary_key_prefix = ''
        for tn in tnarr:
            primary_key_prefix += tn[:1]
        return primary_key_prefix + 'id'

    @staticmethod
    def get_primary_key_field_constraint() -> list:
        return ['NOT NULL', 'UNIQUE']

    def get_primary_key_field_func_decr(self, table_name: str) -> str:
        primary_key_name = self.get_primary_key_field_name(table_name)
        return '\tPRIMARY KEY(\"{}\" AUTOINCREMENT)'.format(primary_key_name)

    def get_field_constraint(self, type_name: str, required: bool = False) -> list:
        constraint = list()
        constraint.append(self.get_type(type_name))
        if required:
            constraint.append('NOT NULL')
        constraint.append(self.get_default_val(type_name))

        return constraint

    def get_primary_key_field_decr(self, table_name: str) -> str:
        primary_key_name = self.get_primary_key_field_name(table_name)

        primary_key_type_arr = list()
        primary_key_type_arr.append(self.get_type('int'))
        primary_key_type_arr += self.get_primary_key_field_constraint()

        return self.get_field_decr(
            primary_key_name,
            primary_key_type_arr
        )

    def get_foreign_key_field_decr(self, foreign_key_name: str) -> str:
        return self.get_field_decr(
            foreign_key_name,
            self.get_field_constraint(
                "int", True
            )
        )

    def get_deleted_field_decr(self) -> str:
        return self.get_field_decr(
            'deleted',
            self.get_field_constraint('bool', True)
        )

    @staticmethod
    def get_field_decr(field_name: str, type_list: list) -> str:
        return '\t\"{}\"    {}'.format(
            field_name,
            ' '.join(type_list)
        )

    def get_create_table_query(self, table_name: str, field_dict: OrderedDict, foreign_key_name: list = None):
        query_str = 'CREATE TABLE IF NOT EXISTS \"{}\" (\n{}\n);'

        field_list = list()
        field_list.append(self.get_primary_key_field_decr(table_name))

        if foreign_key_name is not None:
            for fk in foreign_key_name:
                field_list.append(self.get_foreign_key_field_decr(fk))

        for fk in field_dict:
            field_list.append(
                self.get_field_decr(
                    fk,
                    self.get_field_constraint(
                        field_dict[fk][1], field_dict[fk][0]
                    )
                )
            )

        field_list.append(self.get_deleted_field_decr())
        field_list.append(self.get_primary_key_field_func_decr(table_name))

        return query_str.format(
            table_name,
            ',\n'.join(field_list)
        )

    def create_table(self, table_name: str, field_dict: OrderedDict, foreign_key_name: list = None):
        query = self.get_create_table_query(table_name, field_dict, foreign_key_name)
        self.exec_query(query)
        self.logger.log_info(self.filename, query, visible_tag=False)

    @staticmethod
    def get_insert_record_query(table_name: str, fields: list = None, values_list: list = None):

        query_list = list()

        if fields is not None and values_list is not None:
            query_str = 'INSERT INTO \"{}\" ({}) VALUES ({});'
            fields_str = ', '.join(fields)

            for v_rec in values_list:

                v_list = list()

                for v in v_rec:
                    if type(v).__name__ != 'str' and type(v).__name__ != 'bool':
                        v_list.append('{}'.format(v))
                    elif type(v).__name__ == 'bool':
                        v_list.append('{}'.format(1 if v else 0))
                    else:
                        v_list.append("\"{}\"".format(v))

                query_list.append(query_str.format(table_name, fields_str, ', '.join(v_list)))

            return query_list

        else:
            return None

    def insert_record(self, table_name: str, fields: list = None, values_list: list = None):

        if fields is not None and values_list is not None:
            values_positional_var = ', '.join(['?'] * len(fields))
            query_str = "INSERT INTO \"{}\" ({}) VALUES ({})".format(
                table_name, ', '.join(fields), values_positional_var
            )

            self.exec_query_many(query_str, values_list)
            ql = self.get_insert_record_query(table_name, fields, values_list)

            self.logger.log_info(self.filename, "\n".join(ql), visible_tag=False)

    def exec_query(self, query_str: str):
        cur = self.conn
        cur.execute(query_str)
        if query_str.startswith('INSERT') or \
                query_str.startswith('UPDATE') or \
                query_str.startswith('DELETE'):
            self.conn.commit()

    def exec_query_many(self, query_str: str, values_list: list):
        cur = self.conn
        try:
            cur.executemany(query_str, values_list)
            self.conn.commit()
        except IntegrityError:
            self.conn.rollback()
