#!/usr/bin/python

# python system library
from collections import OrderedDict
from pprint import pformat
import os
import os.path
import subprocess
import sys

# external library
import pandas as pd

# project codes
from data_util import DataUtil
from db_util import DBUtil
from logger_util import LoggerUtil
from refine_data import RefineData


class Main:
    def __init__(self):
        self.data_abspath = ''
        self.result_abspath = ''
        self.koges_abspath = ''
        self.data_relpath = ''
        self.koges_relpath = ''

        self.log_path = ''
        self.log_filename = ''
        self.logger = None

        self.koges_file_dict = dict()
        self.metadata_abspath = ''
        self.metadata_sheet_list = list()
        self.metadata_sheet_dict = OrderedDict()

        self.db = None
        self.db_abspath = ''

        self.examine_df = dict()
        self.person_dict = dict()
        self.examine_number_dict = dict()

        self.init_logger()
        self.init_file_path_dict()
        self.init_db_worker()

    def init_logger(self):
        self.log_path = os.path.join(os.getcwd(), 'logs')
        self.log_filename = 'uric_acid_combine'
        self.logger = LoggerUtil(self.log_path)

    @staticmethod
    def print_separator():
        line_separator = "\n" + "=" * 80 + "\n"
        print(line_separator)
        return line_separator

    def init_file_path_dict(self):
        pwd = os.getcwd()
        program_header = "\nUADB Combinator\n{}\n\n".format("=" * 80)
        print(program_header, end='')
        self.logger.log_info(self.log_filename, program_header, visible_tag=False)

        # need to get file path
        self.data_abspath = os.path.join(pwd, 'data/')
        self.result_abspath = os.path.join(pwd, 'result/')
        self.koges_abspath = os.path.join(self.data_abspath, 'KoGES/')

        print("Data absolute path: {}".format(self.data_abspath))
        self.logger.log_info(self.log_filename,
                             "Data absolute path: {}".format(self.data_abspath))
        print("KoGES absolute path: {}".format(self.koges_abspath))
        self.logger.log_info(self.log_filename,
                             "KoGES absolute path: {}".format(self.koges_abspath))
        print("Result absolute path: {}".format(self.result_abspath))
        self.logger.log_info(self.log_filename,
                             "Result absolute path: {}".format(self.result_abspath))
        print("* Note: Absolute path can be different depending on the environment\n")
        self.logger.log_info(self.log_filename,
                             "* Note: Absolute path can be different"
                             + " depending on the environment\n")

        # need to get rawdata
        self.data_relpath = os.path.join('./', 'data/')
        self.koges_relpath = os.path.join(self.data_relpath, 'KoGES/')
        print("Data relative path: {}".format(self.data_relpath))
        self.logger.log_info(self.log_filename,
                             "Data relative path: {}".format(self.data_relpath))
        print("KoGES relative path: {}".format(self.koges_relpath))
        self.logger.log_info(self.log_filename,
                             "KoGES relative path: {}".format(self.koges_relpath))
        print("* Note: Relative path should be the same although the environment is changed")
        self.logger.log_info(self.log_filename,
                             "* Note: Relative path should be the"
                             + " same although the environment is changed")
        self.logger.log_info(
            self.log_filename,
            self.print_separator()
        )

        self.metadata_abspath = os.path.join(self.data_abspath, 'metadata.xlsx')
        print("Checking {} ... ".format(self.metadata_abspath), end='')
        print("Done!\n" if os.path.exists(self.metadata_abspath) else "Failed!\n")
        if os.path.exists(self.metadata_abspath):
            self.logger.log_info(
                self.log_filename,
                "Checking {} ... Done!\n".format(
                    self.metadata_abspath
                )
            )
        else:
            self.logger.log_error(
                self.log_filename,
                "Checking {} ... Failed!\n".format(self.metadata_abspath)
            )

        if not os.path.exists(self.metadata_abspath):
            sys.exit(1)

        file_list_cmd = "find {} -type f -name '*.csv' | sort"
        koges_file_list_cmd = file_list_cmd.format(os.path.join(self.koges_relpath))
        koges_file_list = [pwd + ps[1:]
                           for ps in subprocess.getoutput(koges_file_list_cmd).split("\n")]

        for f in koges_file_list:
            self.koges_file_dict[f.rsplit('/', 1)[1].split('.')[0]] = f

        for table_name in self.koges_file_dict:
            print("{} = {}".format(table_name, self.koges_file_dict[table_name]))
            self.logger.log_info(
                self.log_filename,
                "{} = {}".format(
                    table_name, self.koges_file_dict[table_name]
                )
            )

        print("\nRAW csv file list is loaded.")
        self.logger.log_info(self.log_filename,
                             "\nRAW csv file list is loaded.")
        self.logger.log_info(
            self.log_filename,
            self.print_separator()
        )

    def init_db_worker(self):
        self.db_abspath = os.path.join(self.result_abspath, 'uric_acid_db.db')
        self.db = DBUtil(self.db_abspath)
        if self.db.get_connection() is not None:
            print("{} is ready.".format(self.db_abspath))
            self.logger.log_info(
                self.log_filename, "{} is ready.".format(self.db_abspath)
            )
        else:
            self.logger.log_error(
                self.log_filename, "{} is not Ready.".format(self.db_abspath)
            )
            raise Exception("{} is not Ready.".format(self.db_abspath))
        
        self.print_separator()

    # noinspection PyTypeChecker
    def get_metadata_sheet_list(self):
        print("Sheets in metadata.xlsx: ", end='')
        sl = DataUtil(self.metadata_abspath, sheet_name=None).get_sheet_list()
        sl = [item for item in sl if not (item.startswith("meta_"))]
        print(sl)
        self.logger.log_info(
            self.log_filename,
            "Sheets in metadata.xlsx: {}".format(sl)
        )
        self.logger.log_info(
            self.log_filename,
            self.print_separator()
        )

        return sl

    def get_metadata_sheet_by_name(self, sheet_name: str = ''):
        du = DataUtil(self.metadata_abspath, sheet_name=sheet_name)
        return du.get_data_frame()

    @staticmethod
    def get_table_schema_from_metadata_sheet(df: pd.DataFrame):
        schema_dict = OrderedDict(dict())

        df_len = len(df)
        for r in range(1, df_len):
            if df.loc[df.index[r], 'include'] == 1.0:
                schema_dict[df.loc[df.index[r], 'common_field_name']] = \
                    [True if df.loc[df.index[r], 'mandatory'] == 1.0 else False,
                     df.loc[df.index[r], 'data_type']]

        return schema_dict

    def load_metadata_sheet(self, sheet_list: list):
        for sn in sheet_list:
            print("Loading {} in metadata.xlsx ... ".format(sn), end='')
            self.metadata_sheet_dict[sn] = self.get_metadata_sheet_by_name(sn)
            print("Done")
            self.logger.log_info(
                self.log_filename,
                "Loading {} in metadata.xlsx ... Done".format(sn)
            )

        self.logger.log_info(
            self.log_filename,
            self.print_separator()
        )

    def create_all_table_if_not_exist(self, sheet_list: list):

        vu_table_name = 'value_units'
        vu_schema_dict = {
            'table_name': [True, "str"],
            'column_name': [True, "str"],
            'description': [True, "str"],
            'value_description': [False, "str"],
            'unit': [False, "str"]
        }

        for sn in sheet_list:
            df = self.metadata_sheet_dict[sn]
            db_table_schema = self.get_table_schema_from_metadata_sheet(df)

            if not sn.startswith('examine'):
                print(self.db.get_create_table_query(sn, db_table_schema, ['epid', 'eiid']))
                self.db.create_table(sn, db_table_schema, ['epid', 'eiid'])
            else:
                print(self.db.get_create_table_query(sn, db_table_schema))
                self.db.create_table(sn, db_table_schema)

        print(self.db.get_create_table_query(vu_table_name, vu_schema_dict))
        self.db.create_table(vu_table_name, vu_schema_dict)

        print("\n{} tables might be created if they do not exist.".format(len(sheet_list) + 1))
        self.logger.log_info(
            self.log_filename,
            "\n{} tables might be created if they do not exist.".format(len(sheet_list) + 1)
        )
        self.logger.log_info(
            self.log_filename,
            self.print_separator()
        )

    def arrange_person_dict_per_area(self, insert_examinee_index_into_db: bool = True):

        tdf = self.metadata_sheet_dict['examinee_person']
        df_len = len(tdf)

        raw_table_list = tdf.columns
        examinee_metadata_dict = dict()

        common_field_name_list = list(tdf.loc[1:, 'common_field_name'])
        value_code_list = list(tdf.loc[1:, 'value_code'])
        data_type_list = list(tdf.loc[1:, 'data_type'])

        # classification based on the area
        for tn in raw_table_list:

            if tn == 'description':
                break

            if tn.startswith('Unnamed'):
                continue

            curr_col_idx = tdf.columns.get_loc(tn)
            common_field_name_idx = tdf.columns.get_loc('common_field_name')
            tn = tn.replace('_cohort_', '_')
            area_cat = tn.split('_')[1]

            for r in range(1, df_len):
                if area_cat not in examinee_metadata_dict:
                    examinee_metadata_dict[area_cat] = dict()

                if pd.isna(tdf.iloc[r, curr_col_idx]):
                    continue

                if tdf.iloc[r, curr_col_idx] not in examinee_metadata_dict[area_cat]:
                    examinee_metadata_dict[area_cat][tdf.iloc[r, curr_col_idx]] = OrderedDict()
                examinee_metadata_dict[area_cat][tdf.iloc[r, curr_col_idx]][
                    tdf.iloc[r, curr_col_idx + 1]
                ] = tdf.iloc[r, common_field_name_idx]

        print(pformat(examinee_metadata_dict))
        self.logger.log_info(
            self.log_filename,
            pformat(examinee_metadata_dict)
        )
        print("Examinee metadata has ready.")
        self.logger.log_info(
            self.log_filename,
            "Examinee metadata has ready."
        )
        self.logger.log_info(
            self.log_filename,
            self.print_separator()
        )

        examinee_area_dict = OrderedDict()

        area_list = list(examinee_metadata_dict.keys())
        for area in area_list:

            if area not in examinee_area_dict:
                examinee_area_dict[area] = OrderedDict()
            
            file_list = list(examinee_metadata_dict[area].keys())
            for f in file_list:
                print("Importing {} ... ".format(self.koges_file_dict[f]), end='')

                self.examine_df[f] = DataUtil(self.koges_file_dict[f]).get_data_frame()
                df_len = len(self.examine_df[f])
                keys_list = list(examinee_metadata_dict[area][f].keys())
                for r in range(df_len):
                    if self.examine_df[f].loc[r, 'DIST_ID'] not in examinee_area_dict[area]:
                        examinee_area_dict[area][self.examine_df[f].loc[r, 'DIST_ID']] = \
                            self.examine_df[f].loc[r, keys_list[1]]

                print("Done")
                self.logger.log_info(
                    self.log_filename,
                    "Importing {} ... Done".format(self.koges_file_dict[f])
                )

        print(pformat(examinee_area_dict))
        self.logger.log_info(
            self.log_filename,
            pformat(examinee_area_dict)
        )
        print("Examinee data has arranged.")
        self.logger.log_info(
            self.log_filename,
            "Examinee data has arranged."
        )
        self.logger.log_info(
            self.log_filename,
            self.print_separator()
        )

        a = 1
        values_list = list()

        elen = 0
        for area in area_list:
            elen += len(list(examinee_area_dict[area].keys()))

        for area in area_list:
            for pid in examinee_area_dict[area]:

                if a % 500 == 0 or a == elen:
                    print("Processing {}/{} for arranging examinee list ...".format(a, elen))
                    self.logger.log_info(
                        self.log_filename,
                        "Processing {}/{} for arranging examinee list ...".format(a, elen)
                    )

                values_list.append(
                    (
                        RefineData.calibrate_data_type(
                            RefineData.get_refined_value(
                                value_code_list[0], pid
                            ),
                            data_type_list[0]
                        ),
                        RefineData.calibrate_data_type(
                            RefineData.get_refined_value(
                                value_code_list[1], area
                            ),
                            data_type_list[1]
                        ),
                        RefineData.calibrate_data_type(
                            RefineData.get_refined_value(
                                value_code_list[2],
                                examinee_area_dict[area][pid]
                            ),
                            data_type_list[2]
                        )
                    )
                )

                self.person_dict['{}_{}'.format(pid, area)] = a
                a += 1

        if insert_examinee_index_into_db:
            self.db.insert_record('examinee_person', common_field_name_list, values_list)
            print("\n{} records are inserted into the {}".format(len(values_list), 'examinee_person'))
            self.logger.log_info(
                self.log_filename,
                "\n{} records are inserted into the {}".format(len(values_list), 'examinee_person')
            )
            self.logger.log_info(
                self.log_filename,
                self.print_separator()
            )

    def arrange_examine_number_dict(self, insert_examinee_index_into_db: bool = True):
        tdf = self.metadata_sheet_dict['examine_id']
        df_len = len(tdf)

        raw_table_list = tdf.columns
        examine_number_metadata_dict = dict()

        common_field_name_list = list(tdf.loc[1:, 'common_field_name'])
        value_code_list = [int(n) for n in list(tdf.loc[1:, 'value_code'])]
        data_type_list = list(tdf.loc[1:, 'data_type'])

        # classification based on the area
        for tn in raw_table_list:

            if tn == 'description':
                break

            if tn.startswith('Unnamed'):
                continue

            curr_col_idx = tdf.columns.get_loc(tn)
            common_field_name_idx = tdf.columns.get_loc('common_field_name')
            area_cat = tn.split('_')[1]
            
            for r in range(1, df_len):
                if area_cat not in examine_number_metadata_dict:
                    examine_number_metadata_dict[area_cat] = OrderedDict()
                
                if pd.isna(tdf.iloc[r, curr_col_idx]):
                    continue
                    
                if tdf.iloc[r, curr_col_idx] not in examine_number_metadata_dict[area_cat]:
                    examine_number_metadata_dict[area_cat][tdf.iloc[r, curr_col_idx]] = OrderedDict()
                examine_number_metadata_dict[area_cat][tdf.iloc[r, curr_col_idx]][
                    tdf.iloc[r, curr_col_idx + 1]
                ] = tdf.iloc[r, common_field_name_idx]

        print(pformat(examine_number_metadata_dict))
        self.logger.log_info(
            self.log_filename,
            pformat(examine_number_metadata_dict)
        )
        print("Examine number metadata has ready.")
        self.logger.log_info(
            self.log_filename,
            "Examine number metadata has ready."
        )
        self.logger.log_info(
            self.log_filename,
            self.print_separator()
        )
        
        values_list = list()
        a = 1

        area_list = list(examine_number_metadata_dict.keys())
        tdf_len = 0
        for area in area_list:

            file_list = list(examine_number_metadata_dict[area].keys())

            for f in file_list:
                df_len = len(self.examine_df[f])
                tdf_len += df_len

                for r in range(df_len):

                    if a % 500 == 0 or a == tdf_len:
                        print("Processing {}/{} for arranging list of examine number ...".format(a, tdf_len))
                        self.logger.log_info(
                            self.log_filename,
                            "Processing {}/{} for arranging list of examine number ...".format(a, tdf_len)
                        )

                    if insert_examinee_index_into_db:
                        en_key = '{}_{}'.format(
                            self.examine_df[f].loc[r, 'DIST_ID'],
                            area
                        )

                        vt_list = list()
                        vt_list.append(self.person_dict[en_key])  # epid
                        vt_list.append(f.split('_')[0])  # cohort category

                        for vk in examine_number_metadata_dict[area][f]:
                            vt_list.append(self.examine_df[f].loc[r, vk])

                        v_list = list()
                        vt_len = len(vt_list)
                        for i in range(vt_len):
                            v_list.append(
                                RefineData.calibrate_data_type(
                                    RefineData.get_refined_value(
                                        value_code_list[i],
                                        vt_list[i]
                                    ),
                                    data_type_list[i]
                                )
                            )

                        values_list.append(tuple(v_list))

                    self.examine_number_dict[
                        '{}_{}'.format(
                            self.examine_df[f].loc[r, 'DIST_ID'],
                            f.split('_')[0]
                        )
                    ] = a

                    a += 1

        print(pformat(self.examine_number_dict))
        self.logger.log_info(
            self.log_filename,
            pformat(self.examine_number_dict)
        )
        print("Examine number data has arranged.\n")
        self.logger.log_info(
            self.log_filename,
            "Examine number data has arranged.\n"
        )

        if insert_examinee_index_into_db:
            self.db.insert_record('examine_id', common_field_name_list, values_list)
            print("{} records are inserted into the {}.".format(len(values_list), 'examine_id'))
            self.logger.log_info(
                self.log_filename,
                "{} records are inserted into the {}.".format(len(values_list), 'examine_id')
            )
            self.logger.log_info(
                self.log_filename,
                self.print_separator()
            )

    def merge_dataset(self, table_name: str = '', insert_data_into_db: bool = True):

        if table_name != '':

            if table_name == 'urine':
                offset_dict = {
                    "NC": 0,
                    "CT": 1,
                    "AS": -1,
                }

            tdf = self.metadata_sheet_dict[table_name]
            tdf_len = len(tdf)
            common_field_name_idx = tdf.columns.get_loc('common_field_name')
            include_idx = tdf.columns.get_loc('include')

            # insert value unit info into the database table
            description_idx = tdf.columns.get_loc('description')
            value_description_idx = tdf.columns.get_loc('value_description')
            value_unit_idx = tdf.columns.get_loc('value_unit')

            value_units_columns = ['table_name', 'column_name', 'description',
                                   'value_description', 'unit']
            value_units_values = list()
            for r in range(1, tdf_len):
                if int(tdf.iloc[r, include_idx]) == 0:
                    continue

                value_units_values.append(
                    (
                        table_name,
                        tdf.iloc[r, common_field_name_idx],
                        tdf.iloc[r, description_idx],
                        tdf.iloc[r, value_description_idx],
                        '' if pd.isna(tdf.iloc[r, value_unit_idx])
                        else tdf.iloc[r, value_unit_idx]
                    )
                )

            if insert_data_into_db:
                self.db.insert_record('value_units', value_units_columns, value_units_values)
                print("{} records are inserted into the {}.".format(len(value_units_values), 'value_units'))
                self.logger.log_info(
                    self.log_filename,
                    "{} records are inserted into the {}.".format(
                        len(value_units_values), 'value_units'
                    )
                )
                self.logger.log_info(
                    self.log_filename,
                    self.print_separator()
                )

            value_code_idx = tdf.columns.get_loc('value_code')
            data_type_idx = tdf.columns.get_loc('data_type')

            raw_table_list = tdf.columns
            metadata_dict = dict()

            for tn in raw_table_list:

                if tn == 'description':
                    break

                if tn.startswith('Unnamed'):
                    continue

                curr_col_idx = tdf.columns.get_loc(tn)
                for r in range(1, tdf_len):
                    if tn not in metadata_dict:
                        metadata_dict[tn] = OrderedDict()

                    if pd.isna(tdf.iloc[r, curr_col_idx]) \
                            or int(tdf.iloc[r, include_idx]) == 0:
                        continue

                    if tdf.iloc[r, curr_col_idx] not in metadata_dict[tn]:
                        metadata_dict[tn][tdf.iloc[r, curr_col_idx]] = OrderedDict()

                    metadata_dict[tn][tdf.iloc[r, curr_col_idx]][tdf.iloc[r, curr_col_idx + 1]] = [
                        int(tdf.iloc[r, value_code_idx]),
                        tdf.iloc[r, common_field_name_idx],
                        tdf.iloc[r, data_type_idx]
                    ]

            print(pformat(metadata_dict))
            self.logger.log_info(
                self.log_filename,
                pformat(metadata_dict)
            )
            print("Merge rule of {} table has generated.".format(table_name))
            self.logger.log_info(
                self.log_filename,
                "Merge rule of {} table has generated.".format(table_name)
            )
            self.logger.log_info(
                self.log_filename,
                self.print_separator()
            )

            for tn in metadata_dict:
                df_dict = dict()
                value_code_list = [0] * 2
                column_list = ['epid', 'eiid']
                data_type_list = ['int'] * 2

                tnn = tn.replace('_cohort_', '_')
                area = tnn.split('_')[1]
                df_len = 0

                for f in metadata_dict[tn]:
                    df_dict[f] = DataUtil(self.koges_file_dict[f]).get_data_frame()

                    for col in metadata_dict[tn][f]:
                        value_code_list.append(
                            metadata_dict[tn][f][col][0]
                        )
                        column_list.append(
                            metadata_dict[tn][f][col][1]
                        )
                        data_type_list.append(
                            metadata_dict[tn][f][col][2]
                        )

                    if df_len == 0:
                        df_len = len(df_dict[f])

                ldlchol_manual_add = False

                if table_name == 'blood':
                    if 'BLOOD_LDLCHOL' not in column_list:
                        value_code_list.append(42)
                        column_list.append('BLOOD_LDLCHOL')
                        data_type_list.append('float')
                        ldlchol_manual_add = True

                data_values_list = list()
                tchol_idx = 0
                hdl_idx = 0
                tglyc_idx = 0

                for r in range(df_len):

                    if ((r + 1) % 500 == 0 and r != 0) or r == df_len - 1:
                        print("Processing {}/{} for {} into the {} ...".format(r + 1, df_len, tn, table_name))
                        self.logger.log_info(
                            self.log_filename,
                            "Processing {}/{} for {} into the {} ...".format(r + 1, df_len, tn, table_name)
                        )

                    data_values_record = list()

                    f = list(metadata_dict[tn].keys())[0]
                    did_idx = df_dict[f].columns.get_loc('DIST_ID')
                    dist_id = df_dict[f].iloc[r, did_idx]
                    fstr = list(metadata_dict[tn].keys())[0]

                    data_values_record.append(
                        self.person_dict['{}_{}'.format(dist_id, area)]
                    )

                    data_values_record.append(
                        self.examine_number_dict['{}_{}'.format(dist_id, fstr.split('_')[0])]
                    )

                    for f in metadata_dict[tn]:
                        for c in metadata_dict[tn][f]:
                            colidx = df_dict[f].columns.get_loc(c)
                            data_values_record.append(
                                df_dict[f].iloc[r, colidx]
                            )

                    if ldlchol_manual_add:
                        tchol_idx = column_list.index('BLOOD_TCHOL')
                        hdl_idx = column_list.index('BLOOD_HDLCHOL')
                        tglyc_idx = column_list.index('BLOOD_TRIGLYCERIDE')

                        data_values_record.append(0.0)

                    clen = len(column_list)

                    for c in range(2, clen):
                        if value_code_list[c] == 42:
                            data_values_record[c] = \
                                RefineData.calibrate_data_type(
                                    RefineData.get_refined_value(
                                        value_code_list[c],
                                        data_values_record[c],
                                        tchl=data_values_record[tchol_idx],
                                        hdl=data_values_record[hdl_idx],
                                        tg=data_values_record[tglyc_idx]
                                    ),
                                    data_type_list[c]
                                )
                        elif table_name == 'urine' and value_code_list[c] == 39:
                            data_values_record[c] = \
                                RefineData.calibrate_data_type(
                                    RefineData.get_refined_value(
                                        value_code_list[c],
                                        data_values_record[c],
                                        urine_offset=offset_dict[f[:2]]
                                    ),
                                    data_type_list[c]
                                )
                        else:
                            data_values_record[c] = \
                                RefineData.calibrate_data_type(
                                    RefineData.get_refined_value(
                                        value_code_list[c],
                                        data_values_record[c]
                                    ),
                                    data_type_list[c]
                                )

                    data_values_list.append(tuple(data_values_record))

                if insert_data_into_db:
                    self.db.insert_record(table_name, column_list, data_values_list)
                    print("\n{} records are inserted into the {}".format(
                        len(data_values_list),
                        table_name
                    ))
                    self.logger.log_info(
                        self.log_filename,
                        "\n{} records are inserted into the {}".format(
                            len(data_values_list),
                            table_name
                        )
                    )
                    self.logger.log_info(
                        self.log_filename,
                        self.print_separator()
                    )

    def run(self,
            pretend_to_insert_examinee: bool = False,
            pretend_to_insert_data: bool = False,
            skip_tables: list = None):
        # create tables
        sheet_list = self.get_metadata_sheet_list()
        self.load_metadata_sheet(sheet_list)
        self.create_all_table_if_not_exist(sheet_list)

        # arrange examinee info
        for sheet in sheet_list:
            if sheet == 'examinee_person':
                self.arrange_person_dict_per_area(
                    insert_examinee_index_into_db=not pretend_to_insert_examinee
                )
            elif sheet == 'examine_id':
                self.arrange_examine_number_dict(
                    insert_examinee_index_into_db=not pretend_to_insert_examinee
                )
            else:
                if skip_tables is not None:
                    if sheet in skip_tables:
                        self.logger.log_warn(
                            self.log_filename,
                            "{} has skipped due to user's exclusion.".format(sheet)
                        )
                        continue

                self.merge_dataset(
                    sheet,
                    insert_data_into_db=not pretend_to_insert_data
                )

        self.logger.log_info(
            self.log_filename,
            "Table set combination has been completed now!"
        )


if __name__ == '__main__':

    m = Main()
    m.run()
